/*
    Toric Lens Converter
    Copyright © 2019 Harry Whitfield

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Toric Lens Converter - browser version 2.1
    27 May, 2019
    Copyright © 2019 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint this */

/*property
    PI, atan2, axs, convert, cos, cyl, prototype, sin, sph, sqrt, toFixed,
    toString
*/

"use strict";

var print;

function Lens(sph, cyl, axs) {	// constructor
	this.sph = sph;
	this.cyl = cyl || 0;
	this.axs = axs || 0;
}

Lens.prototype.convert = function () {	// convert method
	var sph = this.sph + this.cyl;
	var cyl = -this.cyl;
	var axs = this.axs + 90;
	if (axs >= 180) {
		axs -= 180;
	}
	return new Lens(sph, cyl, axs);
};

Lens.prototype.toString = function () {	// toString method
	return (
		this.sph >= 0
		? "+"
		: ""
	) + this.sph.toFixed(2) + (
		this.cyl !== 0
		? " " + (
			this.cyl >= 0
			? "+"
			: ""
		) + this.cyl.toFixed(2) + " x " + this.axs.toFixed(2)
		: ""
	);
};

function rad(x) {
	return x * Math.PI / 180;
}

function deg(x) {
	return 180 * x / Math.PI;
}

function addLenses(lens1, lens2) {
	var sphere;
	var tmp;
	var c1;
	var c2;
	var alpha;
	var alpha2;
	var beta;
	var mre;
	var c000;
	var c045;
	var sph;
	var cyl;
	var axs;
	var sphZ;
	var cylZ;
	var axsZ;

	sphere = lens1.sph;					// accumulate spherical powers
	lens1.sph = 0;

	sphere += lens2.sph;
	lens2.sph = 0;

	if (lens1.cyl < 0) {				// convert to + cyl format
		lens1 = lens1.convert();
		sphere += lens1.sph;
		lens1.sph = 0;
	}

	if (lens2.cyl < 0) {				// convert to + cyl format
		lens2 = lens2.convert();
		sphere += lens2.sph;
		lens2.sph = 0;
	}

	if (lens2.axs < lens1.axs) {		// swap lens1 and lens2
		tmp = lens1;
		lens1 = lens2;
		lens2 = tmp;
	}									// lens1.axs now < lens2.axs

	c1 = lens1.cyl;						// method 1
	c2 = lens2.cyl;
	alpha = lens2.axs - lens1.axs;
	alpha2 = rad(2 * alpha);
	cyl = Math.sqrt(c1 * c1 + c2 * c2 + 2 * c1 * c2 * Math.cos(alpha2));
	sph = sphere + 0.5 * (c1 + c2 - cyl);
	axs = lens1.axs + 0.5 * deg(Math.atan2(c2 * Math.sin(alpha2), c1 + c2 * Math.cos(alpha2)));
	if (axs < 0) {
		axs += 180;
	}
	if (axs >= 180) {
		axs -= 180;
	}

	print("==== ====");
	print("c1: " + c1);
	print("c2: " + c2);
	print("alpha: " + alpha);
	print("sph: " + sph);
	print("cyl: " + cyl);
	print("axs: " + axs);

	c1 = lens1.cyl;						// method 2
	mre = lens1.sph + 0.5 * c1;
	beta = rad(2 * lens1.axs);
	c000 = c1 * Math.cos(beta);
	c045 = c1 * Math.sin(beta);

	c2 = lens2.cyl;
	mre += lens2.sph + 0.5 * c2;
	beta = rad(2 * lens2.axs);
	c000 += c2 * Math.cos(beta);
	c045 += c2 * Math.sin(beta);

	cylZ = Math.sqrt(c000 * c000 + c045 * c045);
	axsZ = 0.5 * deg(Math.atan2(c045, c000));
	sphZ = sphere + mre - 0.5 * cylZ;

	print("---- ----");
	print("c045: " + c045);
	print("c000: " + c000);
	print("sphZ: " + sphZ);
	print("cylZ: " + cylZ);
	print("axsZ: " + axsZ);

	return new Lens(sph, cyl, axs);
}

function bicylinder(c1, a1, c2, a2) {
	var lens1 = new Lens(0, c1, a1);
	var lens2 = new Lens(0, c2, a2);

	return addLenses(lens1, lens2);
}
