/*
    Toric Lens Converter
    Copyright © 2019 Harry Whitfield

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Toric Lens Converter - version 2.0
    26 May, 2019
    Copyright © 2019 Harry Whitfield
    mailto:g6auc@arrl.net
*/

/*jslint browser, this */

/*global paramOne, paramTwo, paramThree, paramFour,
	Lens, Menu, addLenses, MenuArray, MenuSelectedIndex,
	headingOne, headingTwo, headingThree, headingFour
*/

/*property
    background, convert, focus, inRange, isNaN, keyCode, match, onchange,
    onkeydown, onkeypress, preventDefault, prototype, stopPropagation, style,
    title, toString, value, which
*/

// map custom names onto model names
var sphere = paramOne;
var cylinder = paramTwo;
var axis = paramThree;
var alternate = paramFour;
var sphereHeading = headingOne;
var cylinderHeading = headingTwo;
var axisHeading = headingThree;
var alternateHeading = headingFour;
var modeMenu = Menu[0];
var modePref = MenuArray[0][MenuSelectedIndex[0]];

Number.prototype.inRange = function (a, b) {
	return (this >= a) && (this <= b);
};

function process() {
    var sph;
    var cyl;
    var axs;
    var lens;
    var res;
    var lens1;
    var lens2;
    var regExp = /^\s*([+\-]?\d{1,2}(\.\d{1,2})?)(\s*([+\-]?\d{1,2}(\.\d{1,2})?)\s*x\s*(\d{1,3}(\.\d{1,2})?))?\s*$/;
//                    1             2         2 13   4             5         5 4       6       7         7 63
    var found;

	if (modePref === "Addition") {
        lens1 = sphere.value;
        found = lens1.match(regExp);
        if (found !== null) {
            sph = parseFloat(found[1]);
            if (found[3] !== "") {
                cyl = parseFloat(found[4]);
                axs = parseFloat(found[6], 10);
            } else {
                cyl = 0;
                axis = 180;
            }
         	lens1 = new Lens(sph, cyl, axs);
       	} else {
			axis.value = "";
        	alternate.value = "Invalid input Lens 1.";
        	return;
        }

        lens2 = cylinder.value;
        found = lens2.match(regExp);
        if (found !== null) {
            sph = parseFloat(found[1]);
            if (found[3] !== "") {
                cyl = parseFloat(found[4]);
                axs = parseFloat(found[6], 10);
            } else {
                cyl = 0;
                axis = 180;
            }
        	lens2 = new Lens(sph, cyl, axs);
        } else {
			axis.value = "";
        	alternate.value = "Invalid input Lens 2.";
        	return;
        }

        res = addLenses(lens1, lens2);
        axis.value = res.toString();
        alternate.value = res.convert().toString();
    } else {
        sph = parseFloat(sphere.value);
        cyl = parseFloat(cylinder.value);
        axs = parseFloat(axis.value, 10);

        if (Number.isNaN(sph)) {
            alternate.value = "Invalid Sphere Value";
            return;
        }

        if (!sph.inRange(-15, 15)) {
            alternate.value = "Sphere Value Out of Range";
            return;
        }

        if ((cylinder.value !== "") && Number.isNaN(cyl)) {
            alternate.value = "Invalid Cylinder Value";
            return;
        }

        if ((cylinder.value !== "") && !cyl.inRange(-5, 5)) {
            alternate.value = "Cylinder Value Out of Range";
            return;
        }

        if ((axis.value !== "") && Number.isNaN(axs)) {
            alternate.value = "Invalid Axis Value";
            return;
        }

        if ((axis.value !== "") && !axs.inRange(0, 180)) {
            alternate.value = "Axis Value Out of Range";
            return;
        }

        lens = new Lens(sph, cyl, axs);
        res = lens.convert();
        alternate.value = res.toString();
    }
}

modeMenu.onchange = function () {
    "use strict";
    modePref = modeMenu.value;
    switch (modePref) {
    case "Conversion":
        sphereHeading.value = "Sphere";
        cylinderHeading.value = "Cylinder";
        axisHeading.value = "Axis";
        alternateHeading.value = "Alternate Format";

        sphere.title = "Enter sphere in diopters [-15,+15].";
        cylinder.title = "Enter cylinder in diopters [-5,+5].";
        axis.title = "Enter axis in degrees [0,180], then press RETURN to convert.";
        alternate.title = "Alternate format.";

        //axis.bgColor = paleGreen;
        axis.style.background = "rgba(204, 255, 204, 1.0)";

        sphere.value = "3.75";
        cylinder.value = "-1.25";
        axis.value = "65";
        break;
    case "Addition":
        sphereHeading.value = "Lens 1";
        cylinderHeading.value = "Lens 2";
        axisHeading.value = "Sum";
        alternateHeading.value = "Alternate Format";

        sphere.title = "Enter data for lens 1.";
        cylinder.title = "Enter data for lens 2, then press RETURN to calculate.";
        axis.title = "Sum of the powers of the two lenses.";
        alternate.title = "Sum of the powers of the two lenses.";

        //axis.bgColor = grey;
        axis.style.background = "rgba(192 ,192, 192, 1.0)";

        sphere.value = "+0.0 +2.0 x 30";
        cylinder.value = "+0.0 +3.0 x 60";
        axis.value = "";
        break;
    }

	process();
};

sphere.onkeypress = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 13) {
	    process();
	}
};

sphere.onkeydown = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 9) {
        event.preventDefault();
        event.stopPropagation();
  		cylinder.focus();
	}
};

cylinder.onkeypress = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 13) {
	    process();
	}
};

cylinder.onkeydown = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 9) {
        event.preventDefault();
        event.stopPropagation();
  		axis.focus();
	}
};

axis.onkeypress = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if ((x === 13) && (modePref === "Conversion")) {
	    process();
	}
};

axis.onkeydown = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 9) {
        event.preventDefault();
        event.stopPropagation();
  		sphere.focus();
	}
};

alternate.onkeydown = function (event) {
    "use strict";
	var x = event.which || event.keyCode;

	if (x === 9) {
        event.preventDefault();
        event.stopPropagation();
	}
};

sphere.focus();
